/**
   This class defines the nodes for a generic binary tree data structure.

   Data for each node is stored as an object of type T.
*/

public class BTree<T>
{
   private T element;
   private BTree<T> left;
   private BTree<T> right;

   public BTree(T element)
   {
      this.element = element;
      left  = null;
      right = null;
   }

   public BTree(T element, BTree<T> left, BTree<T> right)
   {
      this.element = element;
      this.left  = left;
      this.right = right;
   }

   public T getElement()
   {
      return element;
   }

   public BTree<T> getLeftTree()
   {
      return left;
   }

   public BTree<T> getRightTree()
   {
      return right;
   }

   public void addLeftTree(BTree<T> tree)
   {
      left = tree;
   }

   public void addRightTree(BTree<T> tree)
   {
      right = tree;
   }

   /**
      Compute the number of subtrees below this node.
   */
   public int degree()
   {
      int result = 0;
      if (  left != null ) result++;
      if ( right != null ) result++;
      return result;
   }

   /**
      Calculate the depth of the tree rooted at this node.
   */
   public int depth()
   {
      int result = 0;
      int depthL = 0;
      int depthR = 0;

      if (left != null)
      {
         depthL = getLeftTree().depth(); // recursion
         result = 1;
      }
      if (right != null)
      {
         depthR = getRightTree().depth(); // recursion
         result = 1;
      }

      if (depthL > depthR)
      {
         result += depthL;
      }
      else
      {
         result += depthR;
      }

      return result;
   }

  /**
      Calculate the size of the tree rooted at this node.
   */
   public int size()
   {
      int sizeL = 0;
      int sizeR = 0;

      if (left != null)
      {
         sizeL = getLeftTree().size(); // recursion
      }
      if (right != null)
      {
         sizeR = getRightTree().size(); // recursion
      }
      return 1 + sizeL + sizeR;
   }

   /**
      Convert the tree rooted at this node into a string from the BTree Language.
   */
   @Override
   public String toString()
   {
      String result = "";
      if (null == left && null == right)
      {
         result += element;
      }
      else
      {
         result += "(" + element;
         if (null != left)
         {
            result += " " + left.toString(); // recursion
         }
         else
         {
            result += " ()";
         }
         if (null != right)
         {
            result += " " + right.toString(); // recursion
         }
         else
         {
            result += " ()";
         }
         result += ")";
      }
      return result;
   }
}
