/*
   This program demonstrates how difficult it is in
   languages like Java (or C or C++) to instantiate
   a complex data structure like a binary tree. Below
   is code that instantiates the following binary tree.

                 a                              1
               /   \                          /   \
              /     \                        /     \
            b         c                    2         3
          /   \        \                 /   \        \
         /     \        \               /     \        \
        d       e        f             4       5        6
                        /                              /
                       /                              /
                      g                              7
                       \                              \
                        \                              \
                         h                              8


   This program assumes that you have installed Graphviz, which includes
   the dot.exe program. You can download Graphviz and dot.exe from the
   following URL. Unzip the zip file to your C:\ drive.

     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class BTree_Example
{
   public static void main(String[] args)
   {
      BTree<String> btree1 =
                    new BTree<>("a",
                        new BTree<>("b",
                            new BTree<>("d"),
                            new BTree<>("e")),
                        new BTree<>("c",
                            null,
                            new BTree<>("f",
                                new BTree<>("g",
                                    null,
                                    new BTree<>("h")),
                                null)) );

      // Print the BTree using the BTree's toString() method.
      System.out.println("Binary tree expression is " + btree1 + "\n" );

      // The "pretty printer" prints out the tree in a nice format.
      System.out.println( PrettyPrinter.prettyPrint( btree1 ) + "\n" );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "a b d e c f g h".
      System.out.println( " Pre-order traversal = " + Traverse.preOrder( btree1 ) + "\n" );
      // We should get "d b e a c g h f".
      System.out.println( "  In-order traversal = " + Traverse.inOrder( btree1 ) + "\n" );
      // We should get "d e b h g f c a".
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree1 ) + "\n" );

      // Create dot and png files from the btree.
      if(true)  // "conditional complilation" the java way
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_1";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( BTree2dot.btree2dot(btree1) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }




      BTree<Integer> btree2 =
              new BTree<>(1, new BTree<>(2, new BTree<>(4),
                                            new BTree<>(5)),
                             new BTree<>(3, null,
                                            new BTree<>(6, new BTree<>(7, null,
                                                                          new BTree<>(8)),
                                                           null)) );


      // Print the BTree using the BTree's toString() method.
      System.out.println("Binary tree expression is " + btree2 + "\n" );

      // The "pretty printer" prints out the tree in a nice format.
      System.out.println( PrettyPrinter.prettyPrint( btree2 ) + "\n" );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      System.out.println( " Pre-order traversal = " + Traverse.preOrder(  btree2 ) + "\n" );
      System.out.println( "  In-order traversal = " + Traverse.inOrder(   btree2 ) + "\n" );
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree2 ) + "\n" );

      // Create dot and png files from the btree.
      if(true)  // "conditional complilation" the java way
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_2";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( BTree2dot.btree2dot(btree2) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }
   }
}
